# frozen_string_literal: true

require 'slack-ruby-client'

module PeopleGroup
  module Connectors
    class Slack
      def initialize
        ::Slack.configure do |config|
          config.token = ENV['SLACK_API_TOKEN']
        end
        @client = ::Slack::Web::Client.new
      end

      def find_user(email)
        @client.users_lookupByEmail(email: email)
      rescue ::Slack::Web::Api::Errors::SlackError
        nil
      end

      def bamboo_email_lookup_with_fallback(email)
        file_path = File.join(__dir__, '../../../data/email_mapper.yml')
        email_mapper = YAML.load_file(file_path)
        email_hit = email_mapper.find { |mapping| mapping['bamboo_email'] == email.delete_suffix('@gitlab.com') }
        email = "#{email_hit['slack_email']}@gitlab.com" if email_hit

        @client.users_lookupByEmail(email: email)
      rescue ::Slack::Web::Api::Errors::SlackError
        nil
      end

      def find_user_by_id(id)
        @client.users_info(user: id)
      rescue ::Slack::Web::Api::Errors::SlackError
        nil
      end

      def find_user_profile(id:, include_labels: false)
        @client.users_profile_get(user: id, include_labels: include_labels)
      rescue ::Slack::Web::Api::Errors::SlackError
        nil
      end

      def list_public_channels
        channels = @client.channels_list(exclude_archived: true).channels
        channels.reject(&:is_private)&.map(:name)
      end

      def link_for_message(channel:, timestamp:)
        @client.chat_getPermalink(channel: channel, message_ts: timestamp)
      end

      def send_message(channel:, text: '', as_user: true, attachments: [], thread_ts: nil, reply_broadcast: false, formatted_message: [])
        @client.chat_postMessage(channel: channel, text: text, as_user: true, attachments: attachments, unfurl_links: false, thread_ts: thread_ts, reply_broadcast: reply_broadcast, blocks: formatted_message)
      end

      def update_message(channel:, timestamp:, text: '', attachments: [], formatted_message: [])
        @client.chat_update(channel: channel, text: text, ts: timestamp, as_user: true, attachments: attachments, blocks: formatted_message)
      end

      def send_modal_message(trigger:, view:)
        @client.views_open(trigger_id: trigger, view: view)
      end

      def publish_view(user_id:, trigger:, view:)
        @client.views_publish(user_id: user_id, trigger_id: trigger, view: view)
      end

      def get_message(channel:, timestamp:)
        @client.conversations_history(channel: channel, latest: timestamp, limit: 1, inclusive: true)&.messages&.first
      end
    end
  end
end
